<?php
/**
 * A unique identifier is defined to store the options in the database and reference them from the theme.
 */
function optionsframework_option_name() {
	// Change this to use your theme slug
	return 'options-framework-theme';
}

/**
 * Defines an array of options that will be used to generate the settings page and be saved in the database.
 * When creating the 'id' fields, make sure to use all lowercase and no spaces.
 *
 * If you are making your theme translatable, you should replace 'theme-textdomain'
 * with the actual text domain for your theme.  Read more:
 * http://codex.wordpress.org/Function_Reference/load_theme_textdomain
 */

function optionsframework_options() {

	// Test data
	$test_array = array(
		'one' => __( 'One', 'theme-textdomain' ),
		'two' => __( 'Two', 'theme-textdomain' ),
		'three' => __( 'Three', 'theme-textdomain' ),
		'four' => __( 'Four', 'theme-textdomain' ),
		'five' => __( 'Five', 'theme-textdomain' )
	);

	// Multicheck Array
	$multicheck_array = array(
		'one' => __( 'French Toast', 'theme-textdomain' ),
		'two' => __( 'Pancake', 'theme-textdomain' ),
		'three' => __( 'Omelette', 'theme-textdomain' ),
		'four' => __( 'Crepe', 'theme-textdomain' ),
		'five' => __( 'Waffle', 'theme-textdomain' )
	);

	// Multicheck Defaults
	$multicheck_defaults = array(
		'one' => '1',
		'five' => '1'
	);

	// Background Defaults
	$background_defaults = array(
		'color' => '',
		'image' => '',
		'repeat' => 'repeat',
		'position' => 'top center',
		'attachment'=>'scroll' );

	// Typography Defaults
	$typography_defaults = array(
		'size' => '15px',
		'face' => 'georgia',
		'style' => 'bold',
		'color' => '#bada55' );

	// Typography Options
	$typography_options = array(
		'sizes' => array( '6','12','14','16','20' ),
		'faces' => array( 'Helvetica Neue' => 'Helvetica Neue','Arial' => 'Arial' ),
		'styles' => array( 'normal' => 'Normal','bold' => 'Bold' ),
		'color' => false
	);

	// Pull all the categories into an array
	$options_categories = array();
	$options_categories_obj = get_categories();
	foreach ($options_categories_obj as $category) {
		$options_categories[$category->cat_ID] = $category->cat_name;
	}

	// Pull all tags into an array
	$options_tags = array();
	$options_tags_obj = get_tags();
	foreach ( $options_tags_obj as $tag ) {
		$options_tags[$tag->term_id] = $tag->name;
	}


	// Pull all the pages into an array
	$options_pages = array();
	$options_pages_obj = get_pages( 'sort_column=post_parent,menu_order' );
	$options_pages[''] = 'Select a page:';
	foreach ($options_pages_obj as $page) {
		$options_pages[$page->ID] = $page->post_title;
	}

	// If using image radio buttons, define a directory path
	$imagepath =  get_template_directory_uri() . '/images/';

	$options = array();

	$options[] = array(
		'name' => __( 'General Settings', 'theme-textdomain' ),
		'type' => 'heading'
	);

/*	$options[] = array(
		'name' => __( 'Input Text Mini', 'theme-textdomain' ),
		'desc' => __( 'A mini text input field.', 'theme-textdomain' ),
		'id' => 'example_text_mini',
		'std' => 'Default',
		'class' => 'mini',
		'type' => 'text'
	);

	$options[] = array(
		'name' => __( 'Input Text', 'theme-textdomain' ),
		'desc' => __( 'A text input field.', 'theme-textdomain' ),
		'id' => 'example_text',
		'std' => 'Default Value',
		'type' => 'text'
	);

	$options[] = array(
		'name' => __( 'Input with Placeholder', 'theme-textdomain' ),
		'desc' => __( 'A text input field with an HTML5 placeholder.', 'theme-textdomain' ),
		'id' => 'example_placeholder',
		'placeholder' => 'Placeholder',
		'type' => 'text'
	);

	$options[] = array(
		'name' => __( 'Textarea', 'theme-textdomain' ),
		'desc' => __( 'Textarea description.', 'theme-textdomain' ),
		'id' => 'example_textarea',
		'std' => 'Default Text',
		'type' => 'textarea'
	);

	$options[] = array(
		'name' => __( 'Input Select Small', 'theme-textdomain' ),
		'desc' => __( 'Small Select Box.', 'theme-textdomain' ),
		'id' => 'example_select',
		'std' => 'three',
		'type' => 'select',
		'class' => 'mini', //mini, tiny, small
		'options' => $test_array
	);

	$options[] = array(
		'name' => __( 'Input Select Wide', 'theme-textdomain' ),
		'desc' => __( 'A wider select box.', 'theme-textdomain' ),
		'id' => 'example_select_wide',
		'std' => 'two',
		'type' => 'select',
		'options' => $test_array
	);

	if ( $options_categories ) {
		$options[] = array(
			'name' => __( 'Select a Category', 'theme-textdomain' ),
			'desc' => __( 'Passed an array of categories with cat_ID and cat_name', 'theme-textdomain' ),
			'id' => 'example_select_categories',
			'type' => 'select',
			'options' => $options_categories
		);
	}

	if ( $options_tags ) {
		$options[] = array(
			'name' => __( 'Select a Tag', 'options_check' ),
			'desc' => __( 'Passed an array of tags with term_id and term_name', 'options_check' ),
			'id' => 'example_select_tags',
			'type' => 'select',
			'options' => $options_tags
		);
	}

	$options[] = array(
		'name' => __( 'Select a Page', 'theme-textdomain' ),
		'desc' => __( 'Passed an pages with ID and post_title', 'theme-textdomain' ),
		'id' => 'example_select_pages',
		'type' => 'select',
		'options' => $options_pages
	);

	$options[] = array(
		'name' => __( 'Input Radio (one)', 'theme-textdomain' ),
		'desc' => __( 'Radio select with default options "one".', 'theme-textdomain' ),
		'id' => 'example_radio',
		'std' => 'one',
		'type' => 'radio',
		'options' => $test_array
	);

	$options[] = array(
		'name' => __( 'Example Info', 'theme-textdomain' ),
		'desc' => __( 'This is just some example information you can put in the panel.', 'theme-textdomain' ),
		'type' => 'info'
	);

	$options[] = array(
		'name' => __( 'Input Checkbox', 'theme-textdomain' ),
		'desc' => __( 'Example checkbox, defaults to true.', 'theme-textdomain' ),
		'id' => 'example_checkbox',
		'std' => '1',
		'type' => 'checkbox'
	);

	$options[] = array(
		'name' => __( 'Advanced Settings', 'theme-textdomain' ),
		'type' => 'heading'
	);

	$options[] = array(
		'name' => __( 'Check to Show a Hidden Text Input', 'theme-textdomain' ),
		'desc' => __( 'Click here and see what happens.', 'theme-textdomain' ),
		'id' => 'example_showhidden',
		'type' => 'checkbox'
	);

	$options[] = array(
		'name' => __( 'Hidden Text Input', 'theme-textdomain' ),
		'desc' => __( 'This option is hidden unless activated by a checkbox click.', 'theme-textdomain' ),
		'id' => 'example_text_hidden',
		'std' => 'Hello',
		'class' => 'hidden',
		'type' => 'text'
	);
*/
	$options[] = array(
		'name' => __( 'Header Logo', 'theme-textdomain' ),
		'desc' => __( 'Header Logo', 'theme-textdomain' ),
		'id' => 'header_logo',
		'type' => 'upload'
	);
    $options[] = array(
		'name' => __( 'Header Mail Icon', 'theme-textdomain' ),
		'desc' => __( 'Header Mail Icon', 'theme-textdomain' ),
		'id' => 'header_mail_logo',
		'type' => 'upload'
	);
    $options[] = array(
		'name' => __( 'Header Mail Id', 'theme-textdomain' ),
		'desc' => __( 'Header Mail Id', 'theme-textdomain' ),
		'id' => 'header_mail_id',
		'type' => 'text'
	);
    $options[] = array(
		'name' => __( 'Header Mobile Icon', 'theme-textdomain' ),
		'desc' => __( 'Header Mobile Icon', 'theme-textdomain' ),
		'id' => 'header_mobile_logo',
		'type' => 'upload'
	);
    $options[] = array(
		'name' => __( 'Header Mobile Number', 'theme-textdomain' ),
		'desc' => __( 'Header Mobile Number', 'theme-textdomain' ),
		'id' => 'header_mobile_number',
		'type' => 'text'
	);
	
	$options[] = array(
		'name' => __( 'Floating Whatsapp Link', 'theme-textdomain' ),
		'desc' => __( 'Floating Whatsapp Link', 'theme-textdomain' ),
		'id' => 'floating_wp_number',
		'type' => 'text'
	);
	$options[] = array(
		'name' => __( 'Floating Mobile Number', 'theme-textdomain' ),
		'desc' => __( 'Floating Mobile Number', 'theme-textdomain' ),
		'id' => 'floating_mb_number',
		'type' => 'text'
	);

	
	$options[] = array(
		'name' => __( 'Footer Menu', 'theme-textdomain' ),
		'type' => 'heading'
	);
    
    $options[] = array(
		'name' => __( 'Footer Logo', 'theme-textdomain' ),
		'desc' => __( 'Footer Logo', 'theme-textdomain' ),
		'id' => 'footer_logo',
		'type' => 'upload'
	);

	$wp_editor_settings = array(
		'wpautop' => true, // Default
		'textarea_rows' => 5,
		'tinymce' => array( 'plugins' => 'wordpress,wplink' )
	);

	$options[] = array(
		'name' => __( 'Footer Address', 'theme-textdomain' ),
		'id' => 'footer_address',
		'type' => 'editor',
		'settings' => $wp_editor_settings
	);
    
    $options[] = array(
		'name' => __( 'Facebook Url', 'theme-textdomain' ),
		'id' => 'facebook_url',
		'type' => 'text'
	);
    $options[] = array(
		'name' => __( 'LinkedIn Url', 'theme-textdomain' ),
		'id' => 'linkedin_url',
		'type' => 'text'
	);
    $options[] = array(
		'name' => __( 'Twitter Url', 'theme-textdomain' ),
		'id' => 'twitter_url',
		'type' => 'text'
	);
    $options[] = array(
		'name' => __( 'Youtube Url', 'theme-textdomain' ),
		'id' => 'youtube_url',
		'type' => 'text'
	);
	$options[] = array(
		'name' => __( 'Instagram Url', 'theme-textdomain' ),
		'id' => 'instagram_url',
		'type' => 'text'
	);

	return $options;
}