<?php

class ThemeSupport {
    public function __construct(){
      add_action('init', array($this, 'custom_services_creator'));
      add_action( 'after_setup_theme', array($this,'custom_theme_setup' ));
      add_action('init', array($this, 'custom_testimonials_creator'));
	  add_filter('enter_title_here', array($this,'custom_title_place_holder' ) , 20 , 2 );
      if( function_exists('acf_add_local_field_group') ):  
        add_action ('acf/init', array($this, 'custom_create_acf'));
      endif;
    }
	
	public function custom_title_place_holder($title, $post){
		if( $post->post_type == 'testimonials' ){
            $title = "Add Name";
        }

        return $title;
	}
    
    public function custom_theme_setup(){
        add_theme_support( 'post-thumbnails', array( 'testimonials' ) );
    }
    
    public function custom_testimonials_creator(){
        register_post_type('testimonials',
            array(
                'labels'      => array(
                    'name'          => __('Testimonials', 'textdomain'),
                    'singular_name' => __('Testimonials', 'textdomain'),
                ),
                    'public'      => true,
                    'has_archive' => true,
                    'supports' => array('title', 'thumbnail')
            )
        );
		 register_post_type('claims',
            array(
                'labels'      => array(
                    'name'          => __('Claims', 'textdomain'),
                    'singular_name' => __('Claims', 'textdomain'),
                ),
                    'public'      => true,
                    'has_archive' => true,
                    'supports' => array('title','editor', 'thumbnail')
            )
        );
    }
    
    public function custom_create_acf(){
        acf_add_local_field_group(
         array(
            'key' => 'testimonial_p_options',
            'title' => 'Customer Details',
            'fields' => array (
                  array (
                    'key' => 'designation',
                    'label' => 'Designation',
                    'name' => 'designation',
                    'type' => 'text',
                ),
                array (
                    'key' => 't_comment',
                    'label' => 'Testimonial',
                    'name' => 't_comment',
                    'type' => 'textarea',
                ),
              ),
           'location' => array (
                array (
                    array (
                        "param" =>  "post_type",
                        "operator" =>"==",
                        "value"=> "testimonials"
                    ),
                ),
            ), 
         ));
        acf_add_local_field_group(
           array (
            'key' => 'front_p_options',
            'title' => 'Front Page Options',
            'fields' => array (
                array (
                    'key' => 'home_page_banner_title',
                    'label' => 'Banner Title',
                    'name' => 'home_page_banner_title',
                    'type' => 'text',
                ),
                array (
                    'key' => 'home_page_banner_sub_title',
                    'label' => 'Banner Sub Title',
                    'name' => 'home_page_banner_sub_title',
                    'type' => 'text',
                ),
                array (
                    'key' => 'home_page_banner_video',
                    'label' => 'Banner Video',
                    'name' => 'home_page_banner_video',
                    'type' => 'file',
                    'return_format' => 'url',
                    'mime_type' => 'video/mp4'
                ),
                array (
                    'key' => 'home_page_register_title',
                    'label' => 'Banner Bottom Title',
                    'name' => 'home_page_register_title',
                    'type' => 'text',
                ),
                array (
                    'key' => 'home_page_register_sub_title',
                    'label' => 'Banner Bottom Sub Title',
                    'name' => 'home_page_register_sub_title',
                    'type' => 'text',
                ),
                array (
                    'key' => 'home_page_offer_section_title',
                    'label' => 'Offer Section Title',
                    'name' => 'home_page_offer_section_title',
                    'type' => 'text',
                ),
				array (
                    'key' => 'home_page_offer_section_subtitle',
                    'label' => 'Offer Section Sub Title',
                    'name' => 'home_page_offer_section_subtitle',
                    'type' => 'text',
                ),
                array (
                    'key' => 'home_page_offer_section_button_text',
                    'label' => 'Offer Section Button Text',
                    'name' => 'home_page_offer_section_button_text',
                    'type' => 'text',
                ),
                array (
                    'key' => 'home_page_offer_points_one',
                    'label' => 'Offer Page Points One',
                    'name' => 'home_page_offer_points_one',
                    'type' => 'text',
                ),
                array (
                    'key' => 'home_page_offer_points_two',
                    'label' => 'Offer Page Points Two',
                    'name' => 'home_page_offer_points_two',
                    'type' => 'text',
                ),
                array (
                    'key' => 'home_page_offer_points_three',
                    'label' => 'Offer Page Points Three',
                    'name' => 'home_page_offer_points_three',
                    'type' => 'text',
                ),
                array (
                    'key' => 'home_page_offer_points_four',
                    'label' => 'Offer Page Points Four',
                    'name' => 'home_page_offer_points_four',
                    'type' => 'text',
                ),
                array (
                    'key' => 'home_page_offer_right_title',
                    'label' => 'Offer Page Right Box Title',
                    'name' => 'home_page_offer_right_title',
                    'type' => 'text',
                ),
                array (
                    'key' => 'home_page_offer_right_sub_title',
                    'label' => 'Offer Page Right Sub Title',
                    'name' => 'home_page_offer_right_sub_title',
                    'type' => 'text',
                ),
                array (
                    'key' => 'home_page_offer_title',
                    'label' => 'Offer Page Right SubSection Title',
                    'name' => 'home_page_offer_title',
                    'type' => 'text',
                ),
                array (
                    'key' => 'home_page_offer_sub_title',
                    'label' => 'Offer Page Right SubSection Subtitle',
                    'name' => 'home_page_offer_sub_title',
                    'type' => 'text',
                ),
                array (
                    'key' => 'home_page_offer_counter',
                    'label' => 'Offer Page Right Counter',
                    'name' => 'home_page_offer_counter',
                    'type' => 'text',
                ),
                array (
                    'key' => 'home_page_service_section_title',
                    'label' => 'Home Page Section Title',
                    'name' => 'home_page_service_section_title',
                    'type' => 'text',
                ),
                array (
                    'key' => 'home_page_service_section_sub_title',
                    'label' => 'Home Page Section Sub Title',
                    'name' => 'home_page_service_section_sub_title',
                    'type' => 'text',
                ),
                array (
                    'key' => 'home_page_service_center_title',
                    'label' => 'Home Page Service Section  Title',
                    'name' => 'home_page_service_center_title',
                    'type' => 'text',
                ),
                array (
                    'key' => 'home_page_service_center_sub_title',
                    'label' => 'Home Page Service Section Sub Title',
                    'name' => 'home_page_service_center_sub_title',
                    'type' => 'text',
                ),
 
                array (
                    'key' => 'home_page_video_space_video',
                    'label' => 'Home Page Video File (MP4)',
                    'name' => 'home_page_video_space_video',
                    'type' => 'file',
                    'return_format' => 'url'
                ),
                 array (
                    'key' => 'home_page_video_space_video_webm',
                    'label' => 'Home Page Video File (Webm)',
                    'name' => 'home_page_video_space_video_webm',
                    'type' => 'file',
                    'return_format' => 'url'
                ),
				array (
                    'key' => 'home_page_video_url',
                    'label' => 'Home Page Video URL',
                    'name' => 'home_page_video_url',
                    'type' => 'text',
                ),
                
                
            ),
            'location' => array (
                array (
                    array (
                        "param" =>  "page_type",
                        "operator" =>"==",
                        "value"=> "front_page"
                    ),
                ),
            ),
			"hide_on_screen" => array(
            "the_content"
        ),   
        ));

    }
    
    /**
    * Book My Tm Service Post Type
    */
    public function custom_services_creator(){
        register_post_type('bookmytm_services',
            array(
                'labels'      => array(
                    'name'          => __('Services', 'textdomain'),
                    'singular_name' => __('Service', 'textdomain'),
                ),
                    'public'      => true,
                    'has_archive' => true,
                    'rewrite'=>array(
                      'slug' => 'service'
                      ),
                    'supports' => array('excerpt','editor','title')
            )
        );
        register_taxonomy(
        'bookmytm_services_categories',  
        'bookmytm_services',            
            array(
                'hierarchical' => true,
                'label' => 'Service Categories', 
                'query_var' => true,
                'rewrite' => array(
                    'slug' => 'service-category',    
                    'with_front' => false  
                )
            )
        );

    }
    
}

$ThemeSupport = new ThemeSupport();